<?php
include_once "/usr/local/esia/public_html/api/api_exception.php";
include_once '/usr/local/esia/public_html/api/lib/curl.class.php';

class ClientApi extends Curl
{
	private $dbconnect;
	private $listdroit;
	private $id;
	private $host;
	private $name;
	private $email;
	private $password;
	private $session_id;
	private $inscription_time;
	private $query_access;
	private $port;
	public function __construct() {
		$this->port= 443;

	}

    /**
	 * Authentification du client sur base de son username et son hash
	 * @param string $host url du serveur
	 * @param string $username login du user
	 * @param string $hash digest du user
	 * @retval si authentification OK return true
	 * @exception BadUserPassException si une erreur survient
	 */
	public function digestauth($host,$username,$hash,$port=443)
	{
		$this->port=$port;
		$base_content = $this->getHttpResponse($host.':'.$this->port.'/api/login/'.$username.'?hash='.$hash,$response_status,$response_fields);
		$data =  $this->extractDataToBaseContent($base_content,$response_status);

		if(!isset($data['code'])) // Client Authentifié
		{
			$this->host = $host;
			$this->name = $username;
			$this->password= $hash;
			$this->session_id = $data['login']['sessionId'];
			return true;
		}
		else
		{
			throw new BadUserPassException();
		}
	}

	/**
	 * Authentification du client sur base de son username et mot de passe
	 * @param string $host url du serveur
	 * @param string $username login du user
	 * @param string $pwd password du user
	 * @retval si authentification OK return true
	 * @exception BadUserPassException si une erreur survient
	 */
	public function authentication($host,$username,$pwd,$port=443)
	{
		$this->port=$port;
		$base_content = $this->getHttpResponse($host.':'.$this->port.'/api/login/'.$username,$response_status,$response_fields);

		if( $response_status == 302 ){
                        if( preg_match('/^([^\/]*[^:]*)(:\d+){0,1}\/api\//',$response_fields['redirect_url'],$matches)){
				$base_content = $this->getHttpResponse($response_fields['redirect_url'],$response_status,$response_fields);
				$this->host=$matches[1];
				$this->port=$response_fields['primary_port'];
				$host=$this->host;
				$port=$this->port;
			}
			else throw new Exception('Unable to understand received 302 : redirect response');
		}

		$data =  $this->extractDataToBaseContent($base_content,$response_status);
		if(!isset($data['code'])) // Si pas d'erreur
		{
			$inscriptionTime = $data['login']['inscriptionTime'];
			$str = $username.$pwd.$inscriptionTime;
			$hash = sha1($str);
            $this->inscription_time = $inscriptionTime;
			return $this->digestauth($host,$username,$hash,$port);
		}
		else
		{
			throw new BadUserPassException();
		}
	}

	/**
	 * Effectue une requete get sur le satellite
	 * @param string $path url du serveur
	 * @param string $param (optionnel) parametre supplementaire dans le cas d'un filter
	 * @retval si requete OK return json. Si erreur return null
	 */
	public function get($path,$param=null)
	{
        $path=$this->myUrlEncode($path);
		if(isset($param) && !empty($param))
		{
			$path .= '?'.http_build_query($param);
		}
	//	$path=$this->myUrlEncode($path);
        //echo $this->host.':'.$this->port.'/api/'.$path;
		$base_content = $this->getHttpResponse($this->host.':'.$this->port.'/api/'.$path,$response_status,$response_fields,'GET');
		$data =  $this->extractDataToBaseContent($base_content,$response_status);
//		echo $base_content;
		if(isset($data) && !empty($data))
		{
			return $data;
		}
		return null;
	}
	/**
	 * Effectue une requete get sur le satellite
	 * @param string $path url du serveur
	 * @param string $param (optionnel) parametre supplementaire dans le cas d'un filter
	 * @retval si requete OK return json. Si erreur return null
	 */
	public function getHttp($path,$param=null)
	{
        $path=$this->myUrlEncode($path);
		if(isset($param) && !empty($param))
		{
			$path .= '?'.http_build_query($param);
		}
	//	$path=$this->myUrlEncode($path);
        //echo $this->host.':'.$this->port.'/api/'.$path;
		$base_content = $this->getHttpResponse($this->host.':'.$this->port.'/'.$path,$response_status,$response_fields,'GET');
		if(isset($base_content) && !empty($base_content))
		{
			return $base_content;
		}
		return null;
	}
	public function post($path,$params)
	{
		$base_content = $this->getHttpResponse($this->host.':'.$this->port.'/api/'.$path,$response_status,$response_fields,'POST',$params);
		$data =  $this->extractDataToBaseContent($base_content,$response_status);
//		echo $base_content;
		if(isset($data) && !empty($data))
		{
			return $data;
		}
		return null;
	}

	/**
	 * Logout de l'utilisateur
	 * @retval si logout OK return true. Si erreur return false
	 */
	public function logout()
	{
		$base_content = $this->getHttpResponse($this->host.':'.$this->port.'/api/logout/',$response_status,$response_fields);
		$data =  $this->extractDataToBaseContent($base_content,$response_status);
		if($data=="Logout ok")
		{
			return true;
		}
		return false;
	}

	private function myUrlEncode($string)
	{
		$entities = array('%21', '%2A', '%27', '%28', '%29', '%3B', '%3A', '%40', '%26', '%3D', '%2B', '%24', '%2C', '%2F', '%3F', '%25', '%23', '%5B', '%5D');
		$replacements = array('!', '*', "'", "(", ")", ";", ":", "@", "&", "=", "+", "$", ",", "/", "?", "%", "#", "[", "]");
		return str_replace($entities, $replacements, urlencode($string));
	}

	private function extractDataToParam($path,$param)
	{
		/*
		alert
		alert/history
		alert/history/days/7

		group_node
		group_node/groupnodeid/1

		node
		node/nodeid/1
		node/groupnodeid/1

		plugin
		plugin/pluginid/118

		service
		service/byservice/176
		service/bynode/13

		state
		state/statenode
		state/stateservice
		*/

		$ret = null;
		if(isset($path) && !empty($path))
		{
			/*switch ($path)
			{
				case 'alert':
					break;

				case 'group_node':
					$ret = $path.'/groupnodeid/'.$param.'/';
					break;

				case 'node':
					$ret = $path.'/nodeid/'.$param.'/';
					break;

				case 'plugin':
					$ret = $path.'/pluginid/'.$param.'/';
					break;

				case 'service':
					break;

				case 'state':
					break;
			}*/
		}
		return $ret;
	}

	/**
	 * Extract le data du base_content
	 * @param json content la réponse de la requete http
	 * @retval si extraxt OK return json.
	 */
	private function extractDataToBaseContent($base_content_raw,$response_status){
        $base_content = json_decode($base_content_raw,true);

        if (empty($base_content) && preg_match('/\<title\>(\d+)\s+([^\<]+)<\/title>/',$base_content_raw,$matches)){
            $error_code = $matches[1];
            $error_string = trim($matches[2]);

            if (preg_match('/[Rr]eason:([^<]+)/',$base_content_raw,$matches)){
                    $message = trim($matches[1]);
            }
            throw new HttpBadRequestException("Unable to get JSON content : $error_code - $error_string, $message");
        }

		$data=$base_content['data'];
		$data=$data[0];
		if($response_status >= 400){
			if( isset($data['code'])){
				$ex = HTTPException::makeException($response_status,$data['code'],$data['message']);
			}
			else{
				$ex = HTTPException::makeException($response_status,0);
			}
			throw $ex;
		}
		return $data;
	}
}
?>
